import re
import socket
from bloonix.heaven import H
from bloonix.validator import validate
from bloonix import rx
import bloonix.utils as utils

VERSION = "1.55"
REVISION = "250527154631"

H.version = "%s_%s" % (VERSION, REVISION)
H.config = validate(H.config, {
    "session_expire_time": {
        "type": str,
        "default": "3h",
        "regex": r"^[\d]+[mhd]\Z"
    },
    "session_refresh_time": {
        "type": str,
        "default": "10m",
        "regex": r"^[\d]+[mhd]\Z"
    },
    "cookie_secure": {
        "type": bool,
        "default": True,
    },
    "cookie_http_only": {
        "type": bool,
        "default": True,
    },
    "add_header": {
        "type": list,
        "force_list": True,
        "default": []
    },
    "login_logger": {
        "type": dict,
        "allow_empty": True,
        "default": {},
    },
    "2fa_mail_from": {
        "type": str,
        "regex": rx.EMAIL,
        "optional": True,
    },
    "2fa_command": {
        "type": str,
        "optional": True,
    },
    "2fa_session_expire_time": {
        "type": int,
        "default": 86400 * 30,
    },
    "demo": {
        "type": str,
        "default": "no",
    },
    # deprecated, now in sys_options.
    # hold for backward compability.
    "bloonix_server_names": {
        "type": str,
        "default": socket.gethostname(),
    },
    # deprecated, will be kicked
    "bloonix_proxy_id": {
        "type": str,
        "default": ""
    },
    # deprecated, will be kicked
    "bloonix_max_services": {
        "type": int,
        "default": 0
    },
    # deprecated, will be kicked
    "bloonix_max_wtrm_services": {
        "type": int,
        "default": 0
    },
    # deprecated, will be kicked
    "bloonix_disallow_system_settings": {
        "type": list,
        "force_list": True,
        "default": []
    },
    "log_stat_times": {
        "type": bool,
        "default": 0,
    },
})

for key in ["session_expire_time", "session_refresh_time"]:
    H.config[key] = utils.mhd_to_seconds(
        H.config[key])

if H.config["2fa_mail_from"] is not None and H.config["2fa_command"] is not None:
    raise Exception(
        "mixin 2fa_mail_from with 2fa_command is not allowed, "
        "please remove one of both parameters")

H.model.autoload()
H.view.autoload()
H.controller.autoload()
H.plugin.autoload()
