from bloonix.heaven import H
from bloonix.heaven.plugin.crud import CRUD

class Controller(CRUD):
    def startup(self):
        self.db = H.model.db.group
        self.add_crud_routes("/config/groups")
        H.route("/config/groups/:id/hosts/member", "group_host_member")
        H.route("/config/groups/:id/hosts/no-member", "group_host_no_member")
        H.route("/config/groups/:id/hosts/:host_id/add", "group_host_add")
        H.route("/config/groups/:id/hosts/:host_id/remove", "group_host_remove")
        H.route("/config/groups/:id/users/member", "group_user_member")
        H.route("/config/groups/:id/users/no-member", "group_user_no_member")
        H.route("/config/groups/:id/users/:user_id/add", "group_user_add")
        H.route("/config/groups/:id/users/:user_id/remove", "group_user_remove")

    def auto(self, opts):
        if "id" in opts:
            group = H.model.db.group.get(opts["id"])
            if not group:
                return H.plugin.error.set("err-600")
            H.cache["group"] = group

        if "host_id" in opts:
            host = H.model.db.host.get(opts["host_id"])
            if not host:
                return H.plugin.error.set("err-600")

        return True

    def list(self, opts):
        ret = H.model.db.group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["group", ["id", "name", "description"]],
            order=["asc", "group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def group_host_member(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            join=["inner", ["host_group", "host.id", "host_group.host_id"]],
            condition=["where", ["host_group.group_id", opts["id"]]],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def group_host_no_member(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            condition=[
                "where", ["host.id", "not in", {
                    "table": "host_group",
                    "columns": "host_id",
                    "condition": [
                        "where", ["host_group.group_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def group_host_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_group.create({
            "group_id": opts["id"],
            "host_id": opts["host_id"]
        });

    def group_host_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_group.delete_by_col(
            "group_id", opts["id"],
            "host_id", opts["host_id"]
        )

    def group_user_member(self, opts):
        ret = H.model.db.user.query(
            query=H.req.get_query_data(),
            count="id",
            table=["user", ["id", "username"]],
            join=["inner", ["user_group", "user.id", "user_group.user_id"]],
            condition=["where", ["user_group.group_id", opts["id"]]],
            order=["asc", "user.username"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def group_user_no_member(self, opts):
        ret = H.model.db.user.query(
            query=H.req.get_query_data(),
            count="id",
            table=["user", ["id", "username"]],
            condition=[
                "where", ["user.id", "not in", {
                    "table": "user_group",
                    "columns": "user_id",
                    "condition": [
                        "where", ["user_group.group_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "user.username"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def group_user_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.user_group.create({
            "group_id": opts["id"],
            "user_id": opts["user_id"]
        });

    def group_user_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.user_group.delete_by_col(
            "group_id", opts["id"],
            "user_id", opts["user_id"]
        )
