from bloonix.heaven import H
import bloonix.timeperiod as timeperiod
import bloonix.utils as utils
from bloonix import timezone
from bloonix.validator import Validator
import bloonix.rx as rx
import re
import time
import copy

class Controller:
    def startup(self):
        H.route("/config/hosts/:id/dependencies", "list")
        H.route("/config/hosts/:id/dependencies/add", "add")
        H.route("/config/hosts/:id/dependencies/:dependency_id", "show")
        H.route("/config/hosts/:id/dependencies/:dependency_id/update", "update")
        H.route("/config/hosts/:id/dependencies/:dependency_id/remove", "remove")

    def auto(self, opts):
        if "dependency_id" in opts:
            H.cache["dependency"] = H.model.db.dependency.by_id_and_host_id(
                opts["dependency_id"], opts["id"])
            if not H.cache["dependency"]:
                return H.plugin.error.set("err-600")
        return True

    def show(self, opts):
        H.stash.data = H.cache["dependency"]

    def list(self, opts):
        H.stash.data = H.model.db.dependency.by_host_id(opts["id"])
        H.stash.total = len(H.stash.data)

    def add(self, opts):
        if not H.plugin.token.check():
            return

        data = H.req.params(["service_id", "status", "on_service_id", "on_status"])
        v = Validator(get_resultset=True)
        v.set({
            "service_id": {
                "type": int
            },
            "on_service_id": {
                "type": int
            },
            "status": {
                "type": list,
                "multioptions": [2,3,4,5],
                "default": [2,3,4,5]
            },
            "on_status": {
                "type": list,
                "multioptions": [1,2,3,4,5],
                "default": [1,5]
            }
        })
        res = v.validate(data)

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        service = H.model.db.service.get(data["service_id"])
        on_service = H.model.db.service.get(data["on_service_id"])

        if not service or not on_service:
            failed = []
            if not service:
                failed.append("service_id")
            if not on_service:
                failed.append("on_service_id")
            return H.plugin.error.set("err-610", failed=failed)

        data["host_id"] = service["host_id"]
        data["on_host_id"] = on_service["host_id"]
        H.model.db.dependency.create(data)

        H.plugin.audit.create(
            target="dependency",
            new_data=data
        )

    def update(self, opts):
        if not H.plugin.token.check():
            return

        data = H.req.params(["status", "on_status"])

        v = Validator(get_resultset=True)
        v.set({
            "status": {
                "type": list,
                "multioptions": [2,3,4,5]
            },
            "on_status": {
                "type": list,
                "multioptions": [1,2,3,4,5]
            }
        })
        res = v.validate(data)

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        H.model.db.dependency.update_by_id(opts["dependency_id"], data)

        H.plugin.audit.update(
            target="dependency",
            old_data=H.cache["dependency"],
            new_data=data
        )

    def remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.dependency.delete_by_id(opts["dependency_id"])

        H.plugin.audit.delete(
            target="dependency",
            old_data=H.cache["dependency"]
        )
