from bloonix.heaven import H
from bloonix.json import json
import re

class Controller:
    def startup(self):
        self.db = H.model.db.sys_options
        H.route("/config/system/settings", "list")
        H.route("/config/system/settings/list", "list")
        H.route("/config/system/settings/options", "options")
        H.route("/config/system/settings/update", "update")

    def auto(self, opts):
        if H.user["role"] != "admin":
            return H.plugin.error.set("err-605")
        return True

    def list(self, opts):
        H.stash.data = self.db.get_all_keys()

    def options(self, opts):
        H.stash.data = self.db.validator.get_options()

    def update(self, opts):
        if not H.plugin.token.check():
            return

        params = self.db.validator.get_params()
        data = H.req.params(params)
        extra_data = {}
        password_policy_special = None

        if ("password_policy_special" in data
                and isinstance(data["password_policy_special"], str)):
            value = data["password_policy_special"]
            password_policy_special = value
            value = re.sub(r"\\", r"\\\\", value)
            value = re.sub("\[", r"\\[", value)
            value = re.sub("]", r"\\]", value)
            value = re.sub("-", r"\\-", value)
            value = re.sub("\^", r"\\^", value)
            data["password_policy_special"] = value

        res = self.db.validator.validate(data, ignore_missing=True)

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        # clone data before json.dumps
        H.stash.data = dict(data)

        # set password_policy_special to it's original value
        if "password_policy_special" in data:
            data["password_policy_special_compiled"] = (
                data["password_policy_special"])
            data["password_policy_special"] = password_policy_special

        if "password_policy" in data:
            data["password_policy"] = json.dumps(data["password_policy"])

        # Pre-check
        for key in data:
            if (H.user["id"] != 1
                    and key in H.config["bloonix_disallow_system_settings"]):
                return H.plugin.error.set("err-699")

        for key in data:
            if key == "version" or key == "plugin_version":
                continue
            self.db.update_by_col("key", key, {"value": data[key]})

        H.plugin.audit.update(
            target=self.db.table,
            new_data=data)
