from bloonix.heaven import H
from bloonix.heaven.plugin.crud import Auto, Delete, Show, Options, Validate, RaiseOnDupError
import bloonix.utils as utils
from bloonix.exception import BloonixDuplicationError

class Controller(Auto, Show, Delete, Options, Validate, RaiseOnDupError):
    def startup(self):
        self.db = H.model.db.user
        H.route("/config/users", "list")
        H.route("/config/users/create", "create")
        H.route("/config/users/:id/update", "update")
        H.route("/config/users/:id/delete", "delete_wrapper")
        H.route("/config/users/:id", "show")
        H.route("/config/users/:id/options", "options_wrapper")
        H.route("/config/users/options", "options_wrapper")

    def auto(self, opts):
        if H.user["role"] != "admin":
            return H.plugin.error.set("err-605")
        return super().auto(opts)

    def list(self, opts):
        ret = H.model.db.user.query(
            query=H.req.get_query_data(),
            count="id",
            table=[
                "user", [
                    "id", "username", "name", "role", "last_login", "locked"
                ]
            ],
            order=["asc", "user.username"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        for row in ret.data:
            row["groups"] = H.model.db.group.by_user_id(row["id"])

        H.stash.data = ret.data
        H.stash.total = ret.total

    def options(self, opts):
        if "id" in opts and opts["id"] == 1:
            H.model.db.user.init(validate_admin_account=True)
        else:
            H.model.db.user.init()
        super().options(opts)

    def create(self, opts):
        H.model.db.user.init()
        data = self.validate(opts)
        if not data:
            return

        if data["username"] == "demo":
            return H.plugin.error.set("err-804")

        password, salt, rounds = utils.pbkdf2_sha512_base64(data["password"])
        del data["password"]

        data["stash"] = '{}'
        data["table_settings"] = '{}'

        try:
            H.stash.data = self.db.create_new_user(data, {
                "password": password,
                "salt": salt,
                "rounds": rounds
            })
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise

    def update(self, opts):
        if opts["id"] == 1 and H.user["id"] != 1:
            return H.plugin.error.set("err-699")
        if opts["id"] == 1:
            H.model.db.user.init(validate_admin_account=True)
        else:
            H.model.db.user.init()
        data = self.validate(opts, ignore_missing=True)
        if not data:
            return

        if opts["id"] == 1:
            for key in ["force_password_change", "role"]:
                if key in data:
                    del data[key]

        if "password" in data:
            password, salt, rounds = utils.pbkdf2_sha512_base64(data["password"])
            del data["password"]
            H.model.db.user_secret.update_by_col("user_id", opts["id"], {
                "password": password,
                "salt": salt,
                "rounds": rounds
            })

        if "locked" in data and data["locked"] == 0:
            data["2fa_tries"] = 0

        try:
            self.db.update_by_id(opts["id"], data)
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise

    def options_wrapper(self, opts):
        self.options(opts)

    def delete_wrapper(self, opts):
        if opts["id"] == 1:
            return H.plugin.error.set("err-615")
        if opts["id"] == H.user["id"]:
            return H.plugin.error.set("err-616")
        self.delete(opts)
