import os
import re
import time
from bloonix.json import json
from bloonix.heaven import H
from bloonix.runcmd import RunCmd

class Controller:
    def startup(self):
        H.route("/config/wtrm/validate/:action", "validate")
        H.route("/config/wtrm/options/:action", "options")
        H.route("/config/wtrm/check/", "check")
        H.route("/config/wtrm/check/quick", "check_quick")

    def auto(self, opts):
        if "action" in opts and opts["action"] not in H.model.db.service_parameter.validate_wtrm_actions:
            return H.plugin.error.set("err-404")
        return True

    def validate(self, opts):
        action = opts["action"]
        validator = H.model.db.service_parameter.validate_wtrm_actions
        res = validator[action].validate()
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)
        H.stash.data = res.data

    def options(self, opts):
        action = opts["action"]
        validator = H.model.db.service_parameter.validate_wtrm_actions
        H.stash.data["options"] = validator[action].get_options()
        H.stash.data["values"] = validator[action].get_defaults()

    def check(self, opts):
        self._check(False)

    def check_quick(self, opts):
        self._check(True)

    def _check(self, quick):
        if not H.plugin.token.check():
            return

        workflow = H.req.param("workflow")
        if not isinstance(workflow, list):
            return H.plugin.error.set("err-610")

        if not H.model.db.service_parameter.validate_wtrm_workflow(workflow):
            return H.plugin.error.set("err-610")

        if "HOME" not in os.environ or os.environ["HOME"] == "/root":
            os.environ["HOME"] = "/var/lib/bloonix"

        if "NODE_PATH" not in os.environ:
            os.environ["NODE_PATH"] = "/var/lib/bloonix/node_modules"

        ret = RunCmd(
            command=[
                "mktemp",
                "-dt",
                "bloonix-puppeteer-profile-%s-XXXXXXXXXX" % int(time.time())
            ])
        if ret.has_failed:
            H.log.error(
                "unable to create temporary profile",
                "directory for puppeteer: %s" % ret.error)
            return H.plugin.error.set("err-604")

        tmpdir = ret.stdout.rstrip()
        if not re.search(r"^/tmp/bloonix-puppeteer-profile-[^\s]", tmpdir):
            H.log.warning("unsafe temporary directory created")
            return H.plugin.error.set("err-610")

        orig_puppeteer_tmp_dir = os.environ.get("PUPPETEER_TMP_DIR", None)
        orig_tmpdir = os.environ.get("TMPDIR", None)
        os.environ["PUPPETEER_TMP_DIR"] = tmpdir
        os.environ["TMPDIR"] = tmpdir

        if os.path.isfile("/usr/lib/bloonix/js/bloonix-wtrm.js"):
            bloonix_wtrm_js = "/usr/lib/bloonix/js/bloonix-wtrm.js"
        elif os.path.isfile("/usr/local/lib/bloonix/js/bloonix-wtrm.js"):
            bloonix_wtrm_js = "/usr/local/lib/bloonix/js/bloonix-wtrm.js"
        else:
            H.log.error("bloonix-wtrm.js not found")
            return H.plugin.error.set("err-604")

        command = [
            "node", bloonix_wtrm_js,
            "--workflow", "stdin",
            "--timeout", 75000]
        if quick == False:
            command.append("--screenshots")

        try:
            ret = RunCmd(
                to_stdin=json.dumps(workflow),
                split_lines=True,
                max_bytes=209715200, # 200 MB
                bufsize=1048576,
                timeout=90,
                command=command,
                ignore_exit_code=True)
        finally:
            if orig_puppeteer_tmp_dir is None:
                del os.environ["PUPPETEER_TMP_DIR"]
            else:
                os.environ["PUPPETEER_TMP_DIR"] = orig_puppeteer_tmp_dir
            if orig_tmpdir is None:
                del os.environ["TMPDIR"]
            else:
                os.environ["TMPDIR"] = orig_tmpdir
            RunCmd(command=["rm", "-rf", tmpdir], timeout=20)

        if len(ret.stderr):
            H.log.warning("wtrm check stderr:", ret.stderr)

        if ret.has_timed_out:
            return H.plugin.error.set("err-622")

        if ret.has_failed:
            return H.plugin.error.set("err-604")

        data = []
        for line in ret.stdout:
            if not re.search(r"^\{.+\}$", line):
                continue
            try:
                data.append(json.loads(line))
            except Exception as err:
                H.log.error("json.loads:", err)
                return H.plugin.error.set("err-604")

        H.stash.data = data
