from bloonix.heaven import H
from bloonix import utils
from bloonix.runcmd import RunCmd
import ipaddress
import json
import os
import re
import time

class Controller:
    def startup(self):
        H.route("/login", "login")
        H.route("/logout", "logout")
        H.route("/two-fa-auth", "two_fa_auth")

    def login(self, opts):
        username = H.req.param("username")
        password = H.req.param("password")
        if username and password:
            user = H.model.db.user.find("username", username)
            if user:
                if user["locked"] == 1:
                    if H.req.is_json:
                        H.plugin.error.set("err-632")
                    else:
                        H.stash.data["error_message"] = H.lang.get("err-632")
                        H.stash.template = "login.html"
                        H.render("jinja")
                    return

                secret = H.model.db.user_secret.find("user_id", user["id"])
                password, salt, rounds = utils.pbkdf2_sha512_base64(
                    password, secret["salt"], secret["rounds"])
                if password == secret["password"]:
                    H.user = user
                    sid = utils.pwgen(128)
                    H.model.db.user_session.clear()
                    H.model.db.user_trusted_browser.clear()
                    H.model.db.user.update_by_id(user["id"], {"last_login": H.time})
                    H.lang.set(user["language"])
                    H.res.cookie(
                        name="lang",
                        value=user["language"],
                        secure=H.config["cookie_secure"])
                    H.res.cookie(
                        name="sid",
                        value=sid,
                        expires=H.config["session_expire_time"],
                        secure=H.config["cookie_secure"])
                    session_data = {
                        "sid": sid,
                        "user_id": user["id"],
                        "expire": H.time + H.config["session_expire_time"],
                        "2fa_code": utils.pwgen(6).lower(),
                        "2fa_success": 0}

                    if H.user["2fa_addr"] and not self.__check_2fa_sid():
                        if H.config["2fa_mail_from"]:
                            self.__check_2fa_mail_from(session_data)
                        elif H.config["2fa_command"]:
                            self.__check_2fa_command(session_data)
                        else:
                            H.log.warning(
                                "the user %s has 2fa enabled," % user["id"],
                                "but neither 2fa_mail_from nor",
                                "2fa_command are set up")
                    else:
                        session_data["2fa_success"] = 2

                    H.model.db.user_session.create(session_data)
                    H.plugin.audit.login()
                    H.stash.data["sid"] = sid
                    if username == "demo":
                        self.mail(
                            "Login as %s from %s"
                            % (username, H.req.remote_addr()),
                            "Username: %s\nIP address: %s"
                            % (username, H.req.remote_addr()))
                    if not H.req.is_json:
                        H.res.redirect("/")
                        H.plugin.error.set("err-302")
                    return

        if username or password:
            if H.req.is_json:
                H.plugin.error.set("err-631")
            else:
                H.stash.data["error_message"] = H.lang.get("err-631")
            self.mail(
                "Failed login from %s" % H.req.remote_addr(),
                "Username: %s" % username)
            # NEVER CHANGE THIS LOG LINE. Fail2ban.
            H.log.warning(
                "failed login from %s user %s"
                % (H.req.remote_addr(), username))

        if not H.req.is_json:
            if H.config["demo"] == "yes":
                H.stash.data["login_username"] = "demo"
                H.stash.data["login_password"] = "N0tTheDr01dsYouAreL00k1ngForXDXD"
            H.stash.template = "login.html"
            H.render("jinja")

    def __check_2fa_mail_from(self, session_data):
        if not re.search(r"@", H.config["2fa_mail_from"]):
            H.log.warning("2fa_mail_from must be an email address")
            return

        if not re.search(r"@", H.user["2fa_addr"]):
            H.log.warning(
                "user", H.user["id"], "has not",
                "a valid mail address configured")
            return

        utils.sendmail(
            sender=H.config["2fa_mail_from"],
            to=H.user["2fa_addr"],
            subject="2fa bloonix webgui",
            message=(
                "Dear user, to log in to the Bloonix WebGUI, "
                "please copy the following code and enter it "
                "into the form field that is displayed in the "
                "WebGUI:\r\n\r\nCode: %s\r\n\r\n"
                "Happy Monitoring :-)\r\n" % session_data["2fa_code"]))

    def __check_2fa_command(self, session_data):
        command = H.config["2fa_command"]

        if not os.path.isfile(command) or not os.access(command, os.X_OK):
            H.log.warning("2fa_command", command, "is not executable")
            return

        try:
            ret = RunCmd(
                to_stdin=json.dumps({
                    "username": H.user["username"],
                    "name": H.user["name"],
                    "language": H.user["language"],
                    "2fa_code": session_data["2fa_code"],
                    "2fa_addr": H.user["2fa_addr"],
                }),
                timeout=30,
                command=[command])
            if ret.has_failed:
                H.log.warning(
                    "2fa_command", command,
                    "failed for user", H.user["id"])
        except Exception as err:
            H.log.warning(
                "2fa_command", command,
                "failed for user", H.user["id"],
                "err:", err)

    def logout(self, opts):
        if H.session_id:
            H.plugin.audit.logout()
            H.model.db.user_session.delete_by_col("sid", H.session_id)
            H.res.cookie(name="sid", value=H.session_id, expires="-1m")
            H.res.cookie(name="lang", value="none", expires="-1m")
            H.res.redirect("/")
            H.plugin.error.set("err-302")

    def mail(self, subject, message):
        if not "login_logger" in H.config:
            return

        config = H.config["login_logger"]

        if (isinstance(config, dict)
                and "mail_from" in config
                and isinstance(config["mail_from"], str)
                and "mail_to" in config
                and isinstance(config["mail_to"], str)):
            utils.sendmail(
                sender=config["mail_from"],
                to=config["mail_to"],
                subject="%s - %s" % (config.get("subject", "Bloonix WebGUI"), subject),
                message=message)

    def two_fa_auth(self, opts):
        code = H.req.param("2fa_code")
        if isinstance(code, str) and code.lower() == H.session["2fa_code"]:
            H.model.db.user.reset_tries(H.user["id"])
            H.model.db.user_session.login_2fa_success(H.session["sid"])

            trust_browser = H.req.param("2fa_trust_browser")
            if not isinstance(trust_browser, (int, None)):
                if (isinstance(trust_browser, str)
                        and re.search(r"^(0|1)\Z", trust_browser)):
                    trust_browser = int(trust_browser)
                else:
                    trust_browser = None

            if trust_browser:
                remote_addr = self.__get_remote_addr()
                sid = utils.pwgen(128)
                expire = int(time.time()) + H.config["2fa_session_expire_time"]
                H.model.db.user_trusted_browser.create(
                    data={
                        "user_id": H.user["id"],
                        "2fa_sid": "%s_%s" % (remote_addr, sid),
                        "expire": expire,
                    })
                H.res.cookie(
                    name="2fa_sid",
                    value=sid,
                    expires=H.config["2fa_session_expire_time"],
                    secure=H.config["cookie_secure"])
        elif H.user["2fa_tries"] >= 4:
            H.log.warning(
                "2fa failed, blocking user %s(%s)"
                % (H.user["id"], H.user["username"]))
            H.model.db.user.block_user(H.user["id"])
            H.plugin.error.set("err-630")
            username = H.user["username"]
            self.mail(
                "2fa login failed - user %s blocked"
                % username,
                "Username: %s\nIP address: %s"
                % (username, H.req.remote_addr()))
        else:
            H.model.db.user.increase_2fa_tries(H.user["id"])
            H.plugin.error.set(
                "err-610",
                failed={
                    "2fa_code": {
                        "status": "err-636"
                    }
                })

    def __get_remote_addr(self):
        remote_addr = H.req.remote_addr()
        ip = ipaddress.ip_address(remote_addr)
        if isinstance(ip, ipaddress.IPv4Address):
            return ".".join(remote_addr.split(".")[:3])
        if isinstance(ip, ipaddress.IPv6Address):
            return ":".join(ip.exploded.split(":")[:4])

    def __check_2fa_sid(self):
        sid = "%s_%s" % (self.__get_remote_addr(), H.req.cookie("2fa_sid"))
        return H.model.db.user_trusted_browser.find(
            "user_id", H.user["id"],
            "2fa_sid", sid)
