from bloonix.heaven import H
import bloonix.json as json
import bloonix.utils as utils
import bloonix.timezone as timezones

class Controller:
    def startup(self):
        self.schema = "user"
        H.route("/user", "show")
        H.route(
            "/user/table-settings/:table(host|service|event|chart)",
            "table_settings")
        H.route("/user/set-lang/:lang(en|de)", "set_lang")
        H.route("/user/set-timezone", "set_timezone")
        H.route("/user/set-password", "set_password")
        H.route("/user/set-style/:style(default|dark)", "set_style")
        H.route("/user/password-options", "password_options")
        H.route("/user/dashboards", "get_dashboards")
        H.route("/user/dashboard/default", "get_default_dashboard")
        H.route("/user/dashboard/create", "create_dashboard")
        H.route("/user/dashboard/:dashboard_id", "get_dashboard")
        H.route("/user/dashboard/:dashboard_id/update", "update_dashboard")
        H.route("/user/dashboard/:dashboard_id/delete", "delete_dashboard")
        H.route("/user/set-nav/:set(hidden|compact|visible)", "set_nav")
        self.allowed_columns_by_table = {
            "host": [
                "id",
                "hostname",
                "description",
                "status",
                "ipaddr1",
                "ipaddr2",
                "ipaddr3",
                "ipaddr4",
                "project_class",
                "location_class",
                "enabled",
                "notification_interval",
                "notification_enabled",
                "country_code",
                "templates",
                "status_count",
            ],
            "service": [
                "id",
                "host_id",
                "hostname",
                "ipaddr1",
                "service_name",
                "command",
                "project_class",
                "location_class",
                "status",
                "status_since",
                "attempt_counter",
                "last_check",
                "enabled",
                "notification_enabled",
                "acknowledged",
                "agent_version",
                "message",
                "tags",
                "host_enabled",
                "host_notification_enabled",
            ],
            "event": [
                "time",
                "host_id",
                "hostname",
                "service_id",
                "service_name",
                "last_status",
                "status",
                "attempts",
                "tags",
                "duration",
                "message",
            ],
            "chart":[
                "host_id",
                "hostname",
                "service_id",
                "service_name",
                "plugin_service",
                "chart_title",
            ]
        }

    def auto(self, opts):
        if "dashboard_id" in opts:
            H.cache["dashboard"] = H.model.db.user_dashboard.get_by_user_and_id(
                H.user["id"], opts["dashboard_id"])
            if not H.cache["dashboard"]:
                return H.plugin.error.set("err-600")
        return True

    def show(self, opts):
        H.stash.data = H.model.db.user.get(
            H.user["id"],
            columns=[
                "id",
                "username",
                "name",
                "role",
                "language",
                "timezone",
                "table_settings",
                "settings",
            ]
        )

    def table_settings(self, opts):
        table_name = opts["table"]
        visible_columns = H.req.jsondata
        table_settings = H.user["table_settings"]

        if visible_columns:
            if not isinstance(visible_columns, list):
                return H.plugin.error.set("err-612")
            allowed_columns = self.allowed_columns_by_table[table_name]
            for column in visible_columns:
                if (not isinstance(column, str)
                        or column not in allowed_columns):
                    return H.plugin.error.set("err-612")
            if H.user["username"] != "demo":
                H.model.db.user.update_setting(
                    H.user["id"], "table_settings",
                    table_name, visible_columns)
        elif table_name in table_settings:
            H.stash.data = table_settings[table_name]
        elif table_name == "host":
            H.stash.data = [
                "id",
                "hostname",
                "status_count",
                "ipaddr1",
                "templates",
                "project_class",
                "enabled",
                "notification_enabled"
            ]
        elif table_name == "service":
            H.stash.data = [
                "id",
                "hostname",
                "service_name",
                "status",
                "attempt_counter",
                "last_check",
                "tags",
                "message",
            ]
        elif table_name == "event":
            H.stash.data = [
                "time",
                "hostname",
                "service_name",
                "last_status",
                "status",
                "attempts",
                "duration",
                "tags",
                "message",
            ]
        elif table_name == "chart":
            H.stash.data = [
                "hostname",
                "service_name",
                "plugin_service",
                "chart_title",
            ]
        else:
            H.plugin.error("err-404")

    def set_lang(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        lang = opts["lang"]
        H.lang.set(lang)
        H.res.cookie("lang", lang)
        H.model.db.user.update_by_id(H.user["id"], {"language": lang})

    def set_timezone(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        timezone = H.req.param("timezone")
        if not isinstance(timezone, str) or timezone not in timezones.zones:
            return H.plugin.error.set("err-612")
        H.model.db.user.update_by_id(H.user["id"], {"timezone": timezone})

    def set_style(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        H.model.db.user.update_by_id(H.user["id"], {"style": opts["style"]})

    def set_password(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")

        failed = []
        current_password = H.req.param("current_password")
        new_password = H.req.param("new_password")
        repeat_new_password = H.req.param("repeat_new_password")

        if not isinstance(current_password, str):
            failed.append("current_password")
        if not isinstance(new_password, str):
            failed.append("new_password")
        if not isinstance(repeat_new_password, str):
            failed.append("repeat_new_password")
        if len(failed):
            return H.plugin.error.set("err-610", failed=failed)

        sys_options = H.model.db.sys_options.get_keys(
            "min_password_length",
            "max_password_length")
        secret = H.model.db.user_secret.find("user_id", H.user["id"])
        password, salt, rounds = utils.pbkdf2_sha512_base64(
            current_password, secret["salt"], secret["rounds"])

        if password != secret["password"]:
            return H.plugin.error.set("err-661")
        if new_password != repeat_new_password:
            return H.plugin.error.set("err-664")
        if new_password == current_password:
            return H.plugin.error.set("err-665")
        if len(new_password) < sys_options["min_password_length"]:
            return H.plugin.error.set("err-662")
        if len(new_password) > sys_options["max_password_length"]:
            return H.plugin.error.set("err-663")
        if not H.plugin.password_policy.check_characters(new_password):
            return H.plugin.error.set("err-666")

        new_password, salt, rounds = utils.pbkdf2_sha512_base64(new_password)
        H.model.db.user_secret.update_by_col("user_id", H.user["id"], {
            "password": new_password,
            "salt": salt,
            "rounds": rounds })
        H.model.db.user.update_by_id(
            H.user["id"],
            {"force_password_change": 0})

    def password_options(self, opts):
        H.stash.data = H.model.db.sys_options.get_keys(
            "min_password_length",
            "max_password_length")
        H.stash.data["password_policy"] = H.plugin.password_policy.must_have_characters()

    def get_dashboards(self, opts):
        H.stash.data = H.model.db.user_dashboard.by_user_id(H.user["id"])

    def get_default_dashboard(self, opts):
        H.stash.data = H.model.db.user_dashboard.get_default(
            H.user["id"],
            H.user["last_opened_dashboard"])

    def get_dashboard(self, opts):
        H.stash.data = H.cache["dashboard"]
        if H.stash.data["id"] != H.user["last_opened_dashboard"]:
            H.model.db.user.update_by_id(
                H.user["id"], {"last_opened_dashboard": H.stash.data["id"]})

    def create_dashboard(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        res = H.model.db.user_dashboard.validator.validate()
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)
        res.data["user_id"] = H.user["id"]
        H.stash.data = H.model.db.user_dashboard.create_dashboard(res.data)

    def update_dashboard(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        res = H.model.db.user_dashboard.validator.validate(ignore_missing=True)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)
        H.model.db.user_dashboard.update_dashboard(
            H.user["id"], opts["dashboard_id"], res.data)
        H.stash.data = H.model.db.user_dashboard.get(opts["dashboard_id"])

    def delete_dashboard(self, opts):
        if not H.plugin.token.check():
            return
        if H.user["username"] == "demo":
            return H.plugin.error.set("err-605")
        H.model.db.user_dashboard.delete_by_id(opts["dashboard_id"])

    def set_nav(self, opts):
        H.model.db.user.update_setting(H.user["id"], "settings", "nav", opts["set"])
