from bloonix.dbi.crud import CRUD
from bloonix.heaven import H
from bloonix.heaven.validator import Validator
import bloonix.utils as utils
from bloonix.genattr import GenAttr
import time

class Event(CRUD):
    def validate(self):
        validator = Validator(once=True)
        validator.set({
            "date": {
                "type": str,
                "isa_unixtime": H.user["timezone"],
                "default": None
            }
        })
        return validator.validate()

    def count_latest_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            count="time",
            table="event",
            condition=[
                "where", ["time", ">=", int(time.time()) - 86400],
                "and", ["host_id", host_id]
            ])
        return self.dbi.fetchone(stmt, bind)["count"]

    def latest_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=[
                "event", [
                    "time",
                    "service_id",
                    "status",
                    "last_status",
                    "duration",
                ],
            ],
            condition=[
                "where", ["time", ">=", int(time.time()) - 86400],
                "and", ["host_id", host_id]
            ],
            order=[
                "desc", "time"
            ])
        return self.dbi.fetchall(stmt, bind)

    def list(self, query_data, from_time=None, host_id=None):
        query = {
            "query": query_data,
            "skip": ["time"],
            "parse_int": ["service_id", "last_status", "status", "duration"],
            "count": ["time"],
            #"count": ["time", "service_id", "status", "last_status", "attempts"],
            #"distinct": True,
            "table": [
                "event", [
                    "time",
                    "service_id",
                    "last_status",
                    "status",
                    "attempts",
                    "tags",
                    "duration",
                    "message",
                    "debug"
                ],
                "service_parameter", [
                    "service_name",
                ]
            ],
            "join": [
                "inner", [
                    "service",
                    "event.service_id",
                    "service.id"
                ],
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
            ],
        }

        if from_time is None:
            # Maybe invalid entries in the events table because
            # of time issues with ntp.
            to_time = int(time.time())
            from_time = to_time - 86400
            condition = [
                "where", ["event.time", ">=", from_time],
                "and", ["event.time", "<=", to_time],
            ]
            if host_id:
                condition.append("and")
                condition.append(["event.host_id", host_id])
            stmt, bind = self.sql.select(
                table=["event", "max(time) as time"],
                condition=condition)
            row = self.dbi.fetchone(stmt, bind)
            if not row or "time" not in row or row["time"] is None:
                return GenAttr(data=[], total=0, success=True)
            query["condition"] = [
                "where", ["event.time", ">=", row["time"] - 86400],
                "and", ["event.time", "<=", row["time"]],
            ]
        else:
            query["condition"] = [
                "where", ["event.time", ">=", from_time],
                "and", ["event.time", "<=", from_time + 86400],
            ]

        query["order"] = ["desc", "time"]

        if host_id:
            query["condition"].append("and")
            query["condition"].append(["event.host_id", host_id])
        else:
            query["table"][1].append("host_id")
            query["parse_int"].append("host_id")
            query["table"].append("host")
            query["table"].append("hostname")
            query["join"].append("inner")
            query["join"].append(["host", "event.host_id", "host.id"])
            query = H.plugin.role.filter_events(**query)

        return self.query(**query)

    #def get_last_event(self):
    #    stmt, bind = self.sql.select(
    #        table="event",
    #        columns="max(time)",
    #        condition=condition)
    #    return self.dbi.fetchone(stmt, bind)

    #def get_last_event_by_host_id(self, host_id):
    #    stmt, bind = self.sql.select(
    #        table="event",
    #        columns="max(time)",
    #        condition=["where", ["host_id", host_id]])
    #    return self.dbi.fetchone(stmt, bind)

    def by_service_id(self, service_id, from_time, to_time):
        stmt, bind = self.sql.select(
            table=["event", ["time", "last_status", "status", "duration", "tags"]],
            condition=[
                "where", ["service_id", service_id],
                "and", ["time", ">=", from_time],
                "and", ["time", "<=", to_time],
            ],
            order=["asc", "time"])
        return self.dbi.fetchall(stmt, bind)

    def min_by_service_id(self, service_id, _time):
        stmt, bind = self.sql.select(
            table="event",
            columns="min(time)",
            condition=[
                "where", ["service_id", service_id],
                "and", ["time", ">", _time],
            ])
        return self.dbi.fetchone(stmt, bind)
