from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator

class HostTemplate(CRUDV):
    def add_host(self, template_id, host_id):
        self.dbi.run_transaction(
            {
                "service_parameter": "write",
                "service": "write",
                "host_template": "write"
            },
            self.__add_host,
            template_id, host_id)

    def __add_host(self, template_id, host_id):
        service_parameters = self.schema.service_parameter.fetchall(
            table=["service_parameter", ["id", "command"]],
            condition=["where", ["template_id", template_id]])
        for service_parameter in service_parameters:
            if service_parameter["command"] == "check-wtrm":
                wtrm = True
            else:
                wtrm = None
            self.schema.service.check_count_services(wtrm=wtrm)
            self.schema.service.create({
                "enabled": 1,
                "acknowledged": 0,
                "notification_enabled": 1,
                "service_parameter_id": service_parameter["id"],
                "host_id": host_id
            })
        self.create({
            "host_id": host_id,
            "template_id": template_id
        })

    def remove_host(self, template_id, host_id):
        self.dbi.run_transaction(
            {
                "service_parameter": "write",
                "service": "write",
                "host_template": "write"
            },
            self.__remove_host,
            template_id, host_id)

    def __remove_host(self, template_id, host_id):
        service_parameters = self.schema.service_parameter.fetchall(
            columns="id",
            condition=["where", ["template_id", template_id]])
        for service_parameter in service_parameters:
            self.schema.service.delete_by_col(
                "host_id", host_id,
                "service_parameter_id", service_parameter["id"])
        self.delete_by_col(
            "host_id", host_id,
            "template_id", template_id)
