from bloonix.dbi.crud import CRUD
from bloonix.heaven.validator import Validator
from bloonix import ccodes
from bloonix import rx
from bloonix import utils

class RegauthHost(CRUD):
    def __init__(self):
        self.v_update = Validator()
        self.v_update.set({
            "hostname": {
                "type": str,
                "regex": rx.HOSTNAME
            },
            "ipaddr1": {
                "type": str,
                "regex": rx.IPADDR
            },
            "ipaddr2": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "sysinfo": {
                "type": str,
                "minlength": 0,
                "maxlength": 400,
                "optional": True
            },
            "country_code": {
                "type": str,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": ccodes.country_as_form(),
                "allow_empty": True,
                "optional": True
            },
            "location_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "project_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "tags": {
                "type": list,
                "maxsize": 20,
                "maxlength": 1000,
                "default": [],
                "regex": r"\w",
                "allow_empty": True
            },
            "variables": {
                "type": dict,
                "function": utils.validate_pv_dict,
                "allow_empty": True,
                "maxlength": 20000,
                "default": {}
            }
        })


    def init_v_add_hosts(self):
        self.v_add_hosts = Validator()
        self.v_add_hosts.set({
            "template_id": {
                "type": list,
                "list_of_dicts_key": "value",
                "list_of_dicts_multioptions": self.schema.template.form_data("name", "id"),
                "allow_empty": True,
            }
        })
        self.v_add_hosts.set({
            "contactgroup_id": {
                "type": list,
                "list_of_dicts_key": "value",
                "list_of_dicts_multioptions": self.schema.contactgroup.form_data("name", "id"),
                "allow_empty": True,
            }
        })
        self.v_add_hosts.set({
            "group_id": {
                "type": list,
                "list_of_dicts_key": "value",
                "list_of_dicts_multioptions": self.schema.group.form_data("name", "id"),
                "allow_empty": True,
            }
        })
        self.v_add_hosts.set({
            "location_group_id": {
                "type": list,
                "list_of_dicts_key": "value",
                "list_of_dicts_multioptions": self.schema.location_group.form_data("name", "id"),
                "allow_empty": True,
            }
        })

    def by_hostname(self, hostname):
        return self.fetchone(condition=["where", ["hostname", hostname]])
