import re
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import ccodes
from bloonix import rx

class Satellite(CRUDV):
    def __init__(self):
        self.validator = Validator()
        self.validator.set({
            "name": {
                "type": str,
                "minlength": 1,
                "maxlength": 50,
            },
            "hostname": {
                "type": str,
                "regex": rx.HOSTNAME,
            },
            "port": {
                "type": int,
                "regex": rx.PORT,
                "default": 5464
            },
            "country": {
                "type": str,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": ccodes.country_as_form()
            },
            "description": {
                "type": str,
                "minlength": 0,
                "maxlength": 100,
                "optional": True
            },
            "authkey": {
                "type": str,
                "regex": r"^(?:secret|.{64,})\Z",
                "maxlength": 500
            }
        })

    def get_ids(self):
        data = []
        for row in self.fetchall(columns="id"):
            data.append(row["id"])
        return data

    def by_id(self):
        data = {}
        for row in self.fetchall():
            data[row["id"]] = row
        return data
