import re
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix.heaven import H

class UserDashboard(CRUDV):
    def __init__(self):
        def func(dashlets):
            for dashlet in dashlets:
                if not isinstance(dashlet, dict):
                    return False
                for key in dashlet:
                    if key not in ["name", "config", "x", "y", "w", "h"]:
                        return False
                if ("name" not in dashlet
                        or not isinstance(dashlet["name"], str)
                        or not re.search(r"^[a-z][a-zA-Z0-9_]{0,50}\Z", dashlet["name"])):
                    return False
                if "config" not in dashlet:
                    dashlet["config"] = {}
                if not isinstance(dashlet["config"], dict):
                    return False
                for key in ["x", "y", "w", "h"]:
                    if key in dashlet and not isinstance(dashlet[key], int):
                        if isinstance(dashlet[key], str) and re.search(r"^\d+\Z", dashlet[key]):
                            dashlet[key] = int(dashlet[key])
                        else:
                            del dashlet[key]
            return True

        self.validator = Validator()
        self.validator.set({
            "name": {
                "type": str,
                "minlength": 1,
                "maxlength": 100,
            },
            "dashlets": {
                "type": list,
                "maxsize": 100,
                "allow_empty": True,
                "function": func,
                "default": [],
            },
        })

    def get_default(self, user_id, last_opened_dashboard):
        ret = self.fetchone(
            condition=[
                "where",["user_id", user_id],
                "and", ["id", last_opened_dashboard],
            ])
        if ret is None:
            ret = self.fetchone(condition=["where", ["user_id", user_id]])
            if ret is None:
                ret = self.create_default_dashboard(user_id)
                self.schema.user.update_by_id(H.user["id"], {"last_opened_dashboard": ret["id"]})
        if ret["id"] != last_opened_dashboard:
            self.schema.user.update_by_id(H.user["id"], {"last_opened_dashboard": ret["id"]})
        return ret

    def by_user_id(self, user_id):
        return self.fetchall(
            condition=["where",["user_id", user_id]],
            order=["asc", "name"],
        )

    def get_by_user_and_id(self, user_id, dashboard_id):
        return self.fetchone(
            condition=[
                "where", ["id", dashboard_id],
                "and",["user_id", user_id],
            ])

    def update_dashboard(self, user_id, dashboard_id, data):
        self.update_by_id(dashboard_id, data)

    def create_dashboard(self, data):
        return self.create_and_get(data)

    def create_default_dashboard(self, user_id, with_monitoring_screen=None):
        return self.create_and_get({
            "user_id": user_id,
            "name": "Main",
            "dashlets": [{"name":"clock","config":{},"x":0,"y":0,"w":2,"h":6},{"name":"serviceStatus","config":{"size":"s6x1","interval":30000},"x":2,"y":0,"w":10,"h":3},{"name":"services","config":{"interval":30000,"visible":["hostname","service_name","status","last_check"],"query":[]},"x":2,"y":3,"w":5,"h":19},{"name":"events","config":{"interval":30000,"visible":["time","hostname","service_name","last_status","status","message"],"query":[]},"x":7,"y":3,"w":5,"h":19},{"name":"serviceCounter","config":{"interval":30000,"title":"Unhandled Services","backgroundColor":"auto","color":"#cc333f","query":["status",">",2,"and","acknowledged","==",0]},"x":0,"y":6,"w":2,"h":2},{"name":"serviceCounter","config":{"interval":30000,"title":"Acknowledged Services","backgroundColor":"auto","color":"#5fc15f","query":["acknowledged","==",1]},"x":0,"y":8,"w":2,"h":2},{"name":"serviceCounter","config":{"interval":30000,"title":"Flapping Services","backgroundColor":"auto","color":"#faa732","query":["is_flapping","==",1]},"x":0,"y":10,"w":2,"h":2},{"name":"serviceCounter","config":{"interval":30000,"title":"Services with downtime","backgroundColor":"auto","color":"#328afa","query":["has_downtime","==",1]},"x":0,"y":12,"w":2,"h":2},{"name":"serviceCounter","config":{"interval":30000,"title":"Notifications of Services disabled","backgroundColor":"auto","color":"#b243ff","query":["notification_enabled","==",0]},"x":0,"y":14,"w":2,"h":2},{"name":"serviceCounter","config":{"interval":30000,"title":"Disabled Services","backgroundColor":"auto","color":"#328afa","query":["enabled","==",0]},"x":0,"y":16,"w":2,"h":2},{"name":"hostCounter","config":{"interval":30000,"title":"Notifications of Hosts disabled","backgroundColor":"auto","color":"#b243ff","query":["notification_enabled","==",0]},"x":0,"y":18,"w":2,"h":2},{"name":"hostCounter","config":{"interval":30000,"title":"Disabled Hosts","backgroundColor":"auto","color":"#328afa","query":["enabled","==",0]},"x":0,"y":20,"w":2,"h":2}]
        })
        if with_monitoring_screen is True:
            self.create({
                "user_id": user_id,
                "name": "Monitoring Screen",
                "dashlets": [{"name":"monitoringScreen","config":{"interval":30000,"query":["acknowledged","==",0]},"x":0,"y":0,"w":12,"h":24}]})
